@@ -118,6 +118,7 @@ use crate::locking_method::*;
 /// overhead.  Buckets by themself are not very expensive thus it is recommended to use a
 /// generous large enough number here.  Think about expected number of concurrenct accesses
 /// times four.
+#[derive(Debug)]
 pub struct CacheDb<K, V, const N: usize>
 where
     K: KeyTraits,
@@ -293,35 +294,38 @@ where
 
     /// The 'cache_target' will only recalculated after this many inserts. Should be in the
     /// lower hundreds.
-    pub fn config_target_cooldown(&self, target_cooldown: u32) {
+    pub fn config_target_cooldown(&self, target_cooldown: u32) -> &Self {
         for bucket in &self.buckets {
             bucket
                 .target_cooldown
                 .store(target_cooldown, Ordering::Relaxed);
         }
+        self
     }
 
     /// Sets the lower limit for the 'cache_target' linear interpolation region.  Some
     /// hundreds to thousands of entries are recommended. Should be less than
     /// 'max_capacity_limit'.
-    pub fn config_min_capacity_limit(&self, min_capacity_limit: usize) {
+    pub fn config_min_capacity_limit(&self, min_capacity_limit: usize) -> &Self {
         for bucket in &self.buckets {
             // divide by N so that each bucket gets its share
             bucket
                 .min_capacity_limit
                 .store(min_capacity_limit / N, Ordering::Relaxed);
         }
+        self
     }
 
     /// Sets the upper limit for the 'cache_target' linear interpolation region.
     /// Should be fine around the maximum expected number of entries.
-    pub fn config_max_capacity_limit(&self, max_capacity_limit: usize) {
+    pub fn config_max_capacity_limit(&self, max_capacity_limit: usize) -> &Self {
         for bucket in &self.buckets {
             // divide by N so that each bucket gets its share
             bucket
                 .max_capacity_limit
                 .store(max_capacity_limit / N, Ordering::Relaxed);
         }
+        self
     }
 
     /// Sets the lower limit for the 'cache_target' in percent at 'max_capacity_limit'. Since
@@ -329,43 +333,49 @@ where
     /// cached items for wasting less memory. Note that this counts against the 'capacity' of
     /// the underlying container, not the stored entries. Recommended values are around 5%,
     /// but may vary on the access patterns. Should be lower than 'max_cache_percent'
-    pub fn config_min_cache_percent(&self, min_cache_percent: u8) {
+    pub fn config_min_cache_percent(&self, min_cache_percent: u8) -> &Self {
         assert!(min_cache_percent < 100);
         for bucket in &self.buckets {
             bucket
                 .min_cache_percent
                 .store(min_cache_percent, Ordering::Relaxed);
         }
+        self
     }
 
     /// Sets the upper limit for the 'cache_target' in percent at 'min_capacity_limit'. When
     /// only few entries are stored in a CacheDb it is reasonable to use a lot space for
     /// caching. Note that this counts against the 'capacity' of the underlying container,
     /// thus it should be not significantly over 60% at most.
-    pub fn config_max_cache_percent(&self, max_cache_percent: u8) {
+    pub fn config_max_cache_percent(&self, max_cache_percent: u8) -> &Self {
         assert!(max_cache_percent < 100);
         for bucket in &self.buckets {
             bucket
                 .max_cache_percent
                 .store(max_cache_percent, Ordering::Relaxed);
         }
+        self
     }
 
     /// Sets the number of entries removed at once when evicting entries from the cache. Since
     /// evicting branches into the code parts for removing the entries and calling their
     /// destructors it is a bit more cache friendly to batch a few such things together.
-    pub fn config_evict_batch(&self, evict_batch: u8) {
+    pub fn config_evict_batch(&self, evict_batch: u8) -> &Self {
         for bucket in &self.buckets {
             bucket.evict_batch.store(evict_batch, Ordering::Relaxed);
         }
+        self
     }
 
     /// Evicts up to number entries. The implementation is pretty simple trying to evict number/N from
     /// each bucket. Thus when the distribution is not optimal fewer elements will be removed.
-    pub fn evict(&self, number: usize) {
+    /// Returns the number of items that got evicted
+    pub fn evict(&self, number: usize) -> usize {
+        let mut evicted = number;
         for bucket in &self.buckets {
-            bucket.evict(number / N, &mut bucket.lock_map());
+            evicted -= bucket.evict(number / N, &mut bucket.lock_map());
         }
+        evicted
     }
 }
 
@@ -460,6 +470,7 @@ mod test {
         init();
         let cdb = CacheDb::<String, String, 16>::new();
 
+        println!("Debug {:?}", &cdb);
         assert!(cdb.get(Blocking, &"foo".to_string()).is_err());
     }
 
